/*
 * $Id:$
 *
 * Copyright (c) 2016, Juniper Networks, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the gnu general public license as published by
 * the free software foundation; either version 2, or (at your option)
 * any later version.
 *
 * You should have received a copy of the gnu general public license
 * (for example copying); if not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DRIVETHRES_H_
#define DRIVETHRES_H_

#define DRIVETHRES_H_CVSID \
"$id: drivethres.h 3459 2014-07-26 21:10:37z chrfranke $\n"

#include "atacmds.h"

// Flags state
enum
{
  THRES_INVALID = 0x00,   // User has configured Threshold
  THRES_RAW     = 0x01,   // Compare threshold against Raw value
  THRES_NORM    = 0x02,   // Compare threshold against Normal value
  THRES_LOW     = 0x04,   // Report Error if the value is less than
			  // the configured threshold
  THRES_HIGH    = 0x08,   // Report Error if the value is greater than
			  // the configured threshold
  THRES_VALID   = 0x80
};

// structure to store the threshold configuration for the attribute
struct attr_thres_cfg {
  unsigned char attr_id;
  unsigned int  attr_thres;
  unsigned char attr_flags;

  attr_thres_cfg()
     : attr_id(0) , attr_thres(0) , attr_flags(THRES_INVALID) { }
};

/* The threshold configuration from smartd_thres.conf */
struct ata_smart_threshold_cfg{
  unsigned char id;       // Include ID as well
  unsigned char flags;
  unsigned int threshold;
};

/* The data fields of unigen smart attribute structure are not same
 * as the struct ata_smart_attribute.
 * The attribute Structure for the Unigen SSD's is in the following
 * format:
 * 0 Attribute ID Number
 * 1:2 Status Flag
 * 3:8 Raw Data
 * 9 Value
 * 10 Worst
 * 11 Reserved
 * so the only the data field position of the struct ata_smart_attribute_unigen
 * are changed from struct ata_smart_attribute.
 */
struct ata_smart_attribute_unigen {
    unsigned char id;
    unsigned short status_flag;
    /* note char array used since linux can
     * not handle sizeof short correctly */
    unsigned char raw[6];
    unsigned char normalized;
    unsigned char worse_normal;
    unsigned char reserv;
} __attribute__((packed));

/* From old smartd - for unigen attrib name */
#define VENDOR_ATTRIB_LEN 25

struct ata_vendor_attributes {
    unsigned char id;
    unsigned char vendor_attrib_name[VENDOR_ATTRIB_LEN];
} __attribute__((packed));

char* atagetunigenattribname(unsigned char id);

// Read drive(s) thershold values from configuration file.
int ReadDriveThresConfig();

// Check if the user defined threshold database exists
bool IsThresDbExists();

bool UpdateConfigThersholds(const char *drive_name,
			    const char * model,
			    struct  ata_smart_values * smart_val,
			    struct ata_smart_threshold_cfg * attr_thres);

int GenericCheckSmart(const char *drive_name,
		      const ata_smart_values & curval,
		      struct ata_smart_threshold_cfg * attr_thres,
		      const ata_vendor_attr_defs & defs);

// unigenchecksmart from old smartd (atavendor_db.c)
int UnigenCheckSmart(const char *drive_name,
		     const ata_smart_values & curval,
		     struct ata_smart_threshold_cfg * attr_thres,
		     const ata_vendor_attr_defs & defs);

void set_attr_status();
void send_attr_status();

#endif
